@@ -30,6 +30,8 @@
 #include <unistd.h> // ::link()
 #endif
 
+#include "config-tests.h"
+
 // Debugging notes: to see which inotify signals are emitted, either set s_verboseDebug=true
 // at the top of kdirwatch.cpp, or use the command-line tool "inotifywait -m /path"
 
@@ -77,12 +79,12 @@ class KDirWatch_UnitTest : public QObject
 public:
     KDirWatch_UnitTest()
     {
-        s_staticObjectUsingSelf();
-
-        m_path = m_tempDir.path() + QLatin1Char('/');
         // Speed up the test by making the kdirwatch timer (to compress changes) faster
         qputenv("KDIRWATCH_POLLINTERVAL", "50");
         qputenv("KDIRWATCH_METHOD", KDIRWATCH_TEST_METHOD);
+        s_staticObjectUsingSelf();
+
+        m_path = m_tempDir.path() + QLatin1Char('/');
         KDirWatch *dirW = &s_staticObject()->m_dirWatch;
         m_slow = (dirW->internalMethod() == KDirWatch::FAM || dirW->internalMethod() == KDirWatch::Stat);
         qDebug() << "Using method" << methodToString(dirW->internalMethod());
@@ -91,6 +93,9 @@ public:
 private Q_SLOTS: // test methods
     void initTestCase()
     {
+        QFileInfo pathInfo(m_path);
+        QVERIFY(pathInfo.isDir() && pathInfo.isWritable());
+
         // By creating the files upfront, we save waiting a full second for an mtime change
         createFile(m_path + QLatin1String("ExistingFile"));
         createFile(m_path + QLatin1String("TestFile"));
@@ -112,6 +117,10 @@ private Q_SLOTS: // test methods
     void nestedEventLoop();
     void testHardlinkChange();
     void stopAndRestart();
+    void shouldIgnoreQrcPaths();
+    void benchCreateTree();
+    void benchCreateWatcher();
+    void benchNotifyWatcher();
 
 protected Q_SLOTS: // internal slots
     void nestedEventLoopSlot();
@@ -130,6 +139,7 @@ private:
     void removeFile(int num);
     void appendToFile(const QString &path);
     void appendToFile(int num);
+    int createDirectoryTree(const QString &path, int depth = 4);
 
     QTemporaryDir m_tempDir;
     QString m_path;
@@ -147,9 +157,7 @@ static const int s_maxTries = 50;
 void KDirWatch_UnitTest::createFile(const QString &path)
 {
     QFile file(path);
-    bool ok = file.open(QIODevice::WriteOnly);
-    Q_UNUSED(ok) // silence warnings
-    Q_ASSERT(ok);
+    QVERIFY(file.open(QIODevice::WriteOnly));
     file.write(QByteArray("foo"));
     file.close();
     //qDebug() << path;
@@ -170,6 +178,30 @@ void KDirWatch_UnitTest::removeFile(int num)
     QFile::remove(m_path + fileName);
 }
 
+int KDirWatch_UnitTest::createDirectoryTree(const QString& basePath, int depth)
+{
+    int filesCreated = 0;
+
+    const int numFiles = 10;
+    for (int i = 0; i < numFiles; ++i) {
+        createFile(basePath + QLatin1Char('/') + QLatin1String(s_filePrefix) + QString::number(i));
+        ++filesCreated;
+    }
+
+    if (depth <= 0) {
+        return filesCreated;
+    }
+
+    const int numFolders = 5;
+    for (int i = 0; i < numFolders; ++i) {
+        const QString childPath = basePath + QLatin1String("/subdir") + QString::number(i);
+        QDir().mkdir(childPath);
+        filesCreated += createDirectoryTree(childPath, depth - 1);
+    }
+
+    return filesCreated;
+}
+
 void KDirWatch_UnitTest::waitUntilMTimeChange(const QString &path)
 {
     // Wait until the current second is more than the file's mtime
@@ -256,7 +288,7 @@ QList<QVariantList> KDirWatch_UnitTest::waitForDirtySignal(KDirWatch &watch, int
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDirty.count() << "dirty() signals, expected" << expected;
             return spyDirty;
         }
-        QTest::qWait(50);
+        spyDirty.wait(50);
     }
     return spyDirty;
 }
@@ -273,7 +305,7 @@ bool KDirWatch_UnitTest::waitForOneSignal(KDirWatch &watch, const char *sig, con
                 qWarning() << "Timeout waiting for KDirWatch signal" << QByteArray(sig).mid(1) << "(" << path << ")";
                 return false;
             }
-            QTest::qWait(50);
+            spyDirty.wait(50);
         }
         return verifySignalPath(spyDirty, sig, expectedPath);
     }
@@ -330,7 +362,7 @@ QList<QVariantList> KDirWatch_UnitTest::waitForDeletedSignal(KDirWatch &watch, i
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDeleted.count() << "deleted() signals, expected" << expected;
             return spyDeleted;
         }
-        QTest::qWait(50);
+        spyDeleted.wait(50);
     }
     return spyDeleted;
 }
@@ -512,20 +544,24 @@ void KDirWatch_UnitTest::testDeleteAndRecreateFile() // Useful for /etc/localtim
     //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
     //qDebug() << "initial inode" << stat_buf.st_ino;
 
-    QFile::remove(file1);
-    // And recreate immediately, to try and fool KDirWatch with unchanged ctime/mtime ;)
-    // (This emulates the /etc/localtime case)
-    createFile(file1);
+    // Make sure this even works multiple times, as needed for ksycoca
+    for (int i = 0; i < 5; ++i) {
 
-    //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
-    //qDebug() << "new inode" << stat_buf.st_ino; // same!
+        QFile::remove(file1);
+        // And recreate immediately, to try and fool KDirWatch with unchanged ctime/mtime ;)
+        // (This emulates the /etc/localtime case)
+        createFile(file1);
 
-    {
-        QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
-        if(!waitForRecreationSignal(watch, file1)) {
-            // We may get a dirty signal here instead of a deleted/created set.
-            if (spyDirty.isEmpty() || !verifySignalPath(spyDirty, SIGNAL(dirty(QString)), file1)) {
-                QFAIL("Failed to detect file deletion and recreation through either a deleted/created signal pair or through a dirty signal!");
+        //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
+        //qDebug() << "new inode" << stat_buf.st_ino; // same!
+
+        {
+            QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
+            if(!waitForRecreationSignal(watch, file1)) {
+                // We may get a dirty signal here instead of a deleted/created set.
+                if (spyDirty.isEmpty() || !verifySignalPath(spyDirty, SIGNAL(dirty(QString)), file1)) {
+                    QFAIL("Failed to detect file deletion and recreation through either a deleted/created signal pair or through a dirty signal!");
+                }
             }
         }
     }
@@ -695,6 +731,10 @@ void KDirWatch_UnitTest::testHardlinkChange()
 
     //KDirWatch::statistics();
 
+    // The mtime of the existing file is the one of "TestFile", so it's old.
+    // We won't detect the change then, if we use that as baseline for waiting.
+    // We really need msec granularity, but that requires using statx which isn't available everywhere...
+    waitUntilNewSecond();
     appendToFile(existingFile);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), existingFile));
 #else
@@ -745,4 +785,75 @@ void KDirWatch_UnitTest::stopAndRestart()
     QFile::remove(file3);
 }
 
+void KDirWatch_UnitTest::shouldIgnoreQrcPaths()
+{
+    const auto oldCwd = QDir::currentPath();
+    QVERIFY(QDir::setCurrent(QDir::homePath()));
+
+    KDirWatch watch;
+    watch.addDir(QDir::homePath());
+    // This triggers bug #374075.
+    watch.addDir(QStringLiteral(":/kio5/newfile-templates"));
+
+    QSignalSpy dirtySpy(&watch, &KDirWatch::dirty);
+
+    QFile file(QStringLiteral("bug374075.txt"));
+    QVERIFY(file.open(QIODevice::WriteOnly));
+    QVERIFY(file.write(QByteArrayLiteral("test")));
+    file.close();
+    QVERIFY(file.exists());
+    QVERIFY(dirtySpy.wait());
+    QVERIFY(dirtySpy.count() > 0);
+    QVERIFY(file.remove());
+    QVERIFY(QDir::setCurrent(oldCwd));
+}
+
+void KDirWatch_UnitTest::benchCreateTree()
+{
+#if !ENABLE_BENCHMARKS
+    QSKIP("Benchmarks are disabled in debug mode");
+#endif
+    QTemporaryDir dir;
+
+    QBENCHMARK {
+        createDirectoryTree(dir.path());
+    }
+}
+
+void KDirWatch_UnitTest::benchCreateWatcher()
+{
+#if !ENABLE_BENCHMARKS
+    QSKIP("Benchmarks are disabled in debug mode");
+#endif
+    QTemporaryDir dir;
+    createDirectoryTree(dir.path());
+
+    QBENCHMARK {
+        KDirWatch watch;
+        watch.addDir(dir.path(), KDirWatch::WatchSubDirs | KDirWatch:: WatchFiles);
+    }
+}
+
+void KDirWatch_UnitTest::benchNotifyWatcher()
+{
+#if !ENABLE_BENCHMARKS
+    QSKIP("Benchmarks are disabled in debug mode");
+#endif
+    QTemporaryDir dir;
+    // create the dir once upfront
+    auto numFiles = createDirectoryTree(dir.path());
+    waitUntilMTimeChange(dir.path());
+
+    KDirWatch watch;
+    watch.addDir(dir.path(), KDirWatch::WatchSubDirs | KDirWatch:: WatchFiles);
+
+    // now touch all the files repeatedly and wait for the dirty updates to come in
+    QSignalSpy spy(&watch, &KDirWatch::dirty);
+    QBENCHMARK {
+        createDirectoryTree(dir.path());
+        QTRY_COMPARE_WITH_TIMEOUT(spy.count(), numFiles, s_maxTries * 50 * 2);
+        spy.clear();
+    }
+}
+
 #include "kdirwatch_unittest.moc"
